/**
 * Copyright (c) 2010-2017, Grill Balázs, IncQueryLabs
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *   Grill Balázs - initial API and implementation
 */
package org.eclipse.viatra.query.testing.core.coverage;

import java.util.Collection;
import java.util.List;
import java.util.Set;
import org.eclipse.viatra.query.runtime.api.AdvancedViatraQueryEngine;
import org.eclipse.viatra.query.runtime.api.ViatraQueryMatcher;
import org.eclipse.viatra.query.runtime.exception.ViatraQueryException;
import org.eclipse.viatra.query.runtime.matchers.planning.QueryProcessingException;
import org.eclipse.viatra.query.runtime.matchers.tuple.Tuple;
import org.eclipse.viatra.query.runtime.rete.matcher.ReteBackendFactory;
import org.eclipse.viatra.query.runtime.rete.matcher.ReteEngine;
import org.eclipse.viatra.query.runtime.rete.network.Network;
import org.eclipse.viatra.query.runtime.rete.network.Node;
import org.eclipse.viatra.query.runtime.rete.network.ReteContainer;
import org.eclipse.viatra.query.runtime.rete.network.Supplier;
import org.eclipse.viatra.query.testing.core.coverage.CoverageInfo;
import org.eclipse.viatra.query.testing.core.coverage.CoverageState;
import org.eclipse.xtext.xbase.lib.IterableExtensions;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * This utility can determine the coverage of nodes in a Rete network.
 * 
 * @since 1.6
 */
@SuppressWarnings("all")
public class ReteCoverage {
  private final ReteEngine reteEngine;
  
  /**
   * Although this constructor requires a matcher, it extracts the underlying ReteEngine and
   * the computed coverage is engine-wide.
   */
  public ReteCoverage(final ViatraQueryMatcher<?> matcher) throws ViatraQueryException, QueryProcessingException {
    this(((ReteEngine) ((AdvancedViatraQueryEngine) matcher.getEngine()).getQueryBackend(
      new ReteBackendFactory())));
  }
  
  public ReteCoverage(final ReteEngine reteEngine) {
    this.reteEngine = reteEngine;
  }
  
  public CoverageInfo<Node> reteCoverage() {
    final CoverageInfo<Node> coverage = new CoverageInfo<Node>();
    Network _reteNet = this.reteEngine.getReteNet();
    List<ReteContainer> _containers = _reteNet.getContainers();
    final Procedure1<ReteContainer> _function = new Procedure1<ReteContainer>() {
      @Override
      public void apply(final ReteContainer it) {
        Collection<Node> _allNodes = it.getAllNodes();
        final Procedure1<Node> _function = new Procedure1<Node>() {
          @Override
          public void apply(final Node it) {
            CoverageState _computeCoverage = ReteCoverage.this.computeCoverage(it);
            coverage.put(it, _computeCoverage);
          }
        };
        IterableExtensions.<Node>forEach(_allNodes, _function);
      }
    };
    IterableExtensions.<ReteContainer>forEach(_containers, _function);
    return coverage;
  }
  
  private CoverageState computeCoverage(final Node node) {
    if ((node instanceof Supplier)) {
      CoverageState _xifexpression = null;
      Set<Tuple> _pulledContents = ((Supplier)node).getPulledContents();
      boolean _isEmpty = _pulledContents.isEmpty();
      if (_isEmpty) {
        _xifexpression = CoverageState.NOT_COVERED;
      } else {
        _xifexpression = CoverageState.COVERED;
      }
      return _xifexpression;
    }
    return CoverageState.NOT_REPRESENTED_UNKNOWN_REASON;
  }
}
